import { Body, Controller, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, ServiceError, TransactionService } from 'free-midway-component';
import { GroupService } from '../service/group.service';
import { AqView, CreateDTO_group, CreateVO_group, FetchVO_group, QueryDTO_group, QueryVO_group, UpdateDTO_group, UpdateVO_group, UserListView } from '../interface/group.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { GroupOrderService } from '../service/groupOrder.service';
import { QueryDTO_groupOrder } from '../interface/groupOrder.interface';
import { WxPayService } from '../util/wxpay';
import { WxRefundData } from '../interface/pay.interface';
import _ = require('lodash');
import { randomStr } from '../util/util';
@ApiTags(['团表'])
@Controller('/group')
export class GroupController extends BaseController {
  @Inject()
  service: GroupService;

  @Inject()
  orderService: GroupOrderService;

  @Inject()
  wxpayService: WxPayService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CreateVO_group })
  async create(@Body() data: CreateDTO_group) {
    const result = await this.service.create(data);
    // 暂不需要系统控制团状态
    const status = _.get(data, 'status');
    if (status === '0') await this.service.toSendMsg(result);
    return result;
  }
  @Get('/')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_group })
  async query(@Query() filter: QueryDTO_group, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/aq')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_group })
  async aggQuery(@Query() filter: object) {
    const po = _.pick(filter, ['skip', 'limit']);
    const fo = _.omit(filter, ['skip', 'limit']);
    const { data, total } = await this.service.aggQuery(fo, po);
    const list = [];
    for (const i of data) {
      const d = new AqView(i);
      list.push(d);
    }
    return { data: list, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FetchVO_group })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id, { populate: false });
    const result = new FetchVO_group(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UpdateVO_group })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_group) {
    // 需要检查这个团里有没有订单,如果有订单,需要将订单退款后再解散团
    const { status } = body;
    if (status === '-1') {
      const filter = new QueryDTO_groupOrder();
      filter.props = ['group', 'status'];
      filter.group = id;
      filter.status = '1';
      const orderList = await this.orderService.query(filter);
      // 退单
      const errorList = [];
      for (const o of orderList) {
        try {
          const rd = new WxRefundData();
          rd.order_no = _.get(o, 'pay.pay_no');
          rd.money = _.get(o, 'pay.pay_money');
          rd.out_refund_no = `${_.get(o, 'pay.pay_no')}-r-${randomStr()}`;
          rd.reason = '散团退款';
          // 退款
          await this.wxpayService.refund(rd);
          // 修改订单
          await this.orderService.updateOne(_.get(o, '_id'), { status: '-1' });
        } catch (error) {
          errorList.push(_.get(o, 'no'));
        }
      }
      if (errorList.length > 0) throw new ServiceError(`散团失败! 订单:${errorList.join(';')} 未退款成功,请重新尝试`);
    }
    const result = await this.service.updateOne(id, body);
    // 最后判断是否需要重新发送mq消息
    const msg = await this.service.needMqMsg({ ...body, id });
    if (msg) await this.service.toSendMsg({ id, ...msg });
    if (status === '1' || status === '-1') this.service.toSendCustomerMsg(id);
    return result;
  }

  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }

  @Get('/userView')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: UserListView })
  async userView(@Query() filter: QueryDTO_group, @Query('skip') skip: number, @Query('limit') limit: number) {
    const { data, total } = await this.service.userListView(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const newData = [];
    for (const i of data) {
      const nd = new UserListView(i);
      newData.push(nd);
    }
    return { data: newData, total };
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
