import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { GroupAfterSaleService } from '../service/groupAfterSale.service';
import {
  AdminView,
  CreateDTO_groupAfterSale,
  CreateVO_groupAfterSale,
  QueryDTO_groupAfterSale,
  QueryVO_groupAfterSale,
  UpdateDTO_groupAfterSale,
  UpdateVO_groupAfterSale,
  UserListView,
} from '../interface/groupAfterSale.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { TransactionService } from '../util/transactions';
import _ = require('lodash');
import moment = require('moment');
import { WxPayService } from '../util/wxpay';
import { GroupOrderService } from '../service/groupOrder.service';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { Config } from '../entity/base/config';
import { ReturnModelType } from '@typegoose/typegoose';
import * as computedUtil from '../util/computed';
@ApiTags(['团购售后'])
@Controller('/groupAfterSale')
export class GroupAfterSaleController extends BaseController {
  @Inject()
  service: GroupAfterSaleService;
  @Inject()
  orderService: GroupOrderService;

  @Inject()
  wxpayService: WxPayService;

  @InjectEntityModel(Config)
  configModel: ReturnModelType<typeof Config>;

  @Post('/')
  @ApiResponse({ type: CreateVO_groupAfterSale })
  async create(@Body() data: CreateDTO_groupAfterSale) {
    const order = _.get(data, 'order');
    if (!order) throw new ServiceError('缺少订单信息', FrameworkErrorEnum.NEED_BODY);
    const od = await this.orderService.fetch(order, { populate: false });
    if (!od) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    const has_afterSale = await this.service.findOne({ order, status: { $nin: ['!1', '!2', '!3', '!4', '!5'] } });
    if (has_afterSale) throw new ServiceError('该订单已有正在处理中的售后申请,请勿重复申请', FrameworkErrorEnum.SERVICE_FAULT);
    const customer = _.get(od, 'customer');
    data.customer = customer;
    if (!data.apply_time) data.apply_time = moment().format('YYYY-MM-DD HH:mm:ss');
    const type = data.type;
    if (type === '4' || type === '5') {
      const realPay = _.get(od, 'pay.pay_money');
      data.money = realPay;
      data.desc = type === '4' ? '取消订单' : '拒收商品';
    }
    const result = await this.service.create(data);
    return result;
  }
  @Get('/')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_groupAfterSale })
  async query(@Query() filter: QueryDTO_groupAfterSale, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: AdminView })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id, { populate: true, lean: true });
    const nd = await this.service.addInfo(data);
    const result = new AdminView(nd);
    return result;
  }

  @Post('/:id')
  @ApiResponse({ type: UpdateVO_groupAfterSale })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_groupAfterSale) {
    const asd = await this.service.fetch(id);
    if (!asd) throw new ServiceError('未找到售后数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    const order_id = _.get(asd, 'order._id');
    const tran = new TransactionService();
    try {
      const type = _.get(asd, 'type');
      const status = _.get(body, 'status');
      if (!status) {
        const data = await this.service.updateOne(id, body);
        return data;
      }
      let refundInfo;
      const uStatus = _.get(body, 'status');
      if (type === '1') {
        // 仅退款,退优惠券,退款
        // 如果不是处理中,则不进行退款
        if (_.get(body, 'status') === '1') {
          // 1.检验并组织退款信息
          refundInfo = await this.service.toReturnMoney(asd, body, tran);
          // 3.修改数据, 直接修改成退完款的状态.如果后面退款失败了.直接回滚了
          body.status = '-1';
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
          this.service.refundOrder(order_id, tran);
        } else if (_.get(body, 'status') === '!1') {
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
        }
        // 4.修改订单状态
      } else if (type === '2') {
        // 退货(退库存),退款,退优惠券
        // 做记录,但是不是结束状态,都不退
        if (uStatus === '-2') {
          // 1.检验并组织退款信息
          refundInfo = await this.service.toReturnMoney(asd, body, tran);
          // 2.修改订单状态
          this.service.refundOrder(order_id, tran);
          // 结束时间
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
        } else if (uStatus === '!2') {
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
        }
      } else if (type === '3') {
        // 换货,不需要退款
        // 但需要检查买卖双方的快递是否签收,都签收的话,需要将状态改为结束
        const uto = _.get(body, 'transport', {});
        const eto = _.get(asd, 'transport', {});
        // 有关快递的字段整合,将传来的和以前的放在一起.然后找是否签收
        const to = { ...eto, ...uto };
        const cr = _.get(to, 'customer_receive');
        const sr = _.get(to, 'shop_receive');
        if (cr && sr) {
          body.status = '-3';
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
          this.service.refundOrder(order_id, tran);
        } else if (uStatus === '!3') {
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
        }
      } else if (type === '4') {
        // 取消订单: 退钱,退货(退库存)----不过不需要有快递信息,没发货,退优惠券
        // 没有商品,只有拆分的订单号,用这个去退
        if (uStatus === '4') {
          refundInfo = await this.service.returnOrder(asd, tran);
          body.status = '-4';
          this.service.refundOrder(order_id, tran);
        }
        body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
      } else if (type === '5') {
        // 拒收: 退钱, 退货
        // 如果状态不是完成,那就不退,只是数据修改
        if (uStatus === '-5') {
          refundInfo = await this.service.returnOrder(asd, tran);
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
          this.service.refundOrder(order_id, tran);
        } else if (uStatus === '!5') {
          body.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
        }
      } else throw new ServiceError('未知的售后类型,无法处理', FrameworkErrorEnum.SERVICE_FAULT);
      // 售后处理人的添加
      if (body.status !== '0') {
        const admin = this.ctx.admin;
        body.deal_person = admin._id;
      }
      // 修改数据
      tran.update('GroupAfterSale', { _id: id }, body);
      await tran.run();
      // 退钱
      if (!refundInfo) return;
      const res = await this.wxpayService.refund(refundInfo);
      if (res.errcode && res.errcode !== 0) throw new ServiceError(res.errmsg, FrameworkErrorEnum.SERVICE_FAULT);
    } catch (error) {
      await tran.rollback();
      console.log(error);
      throw new ServiceError('售后处理失败', FrameworkErrorEnum.SERVICE_FAULT);
    }
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }

  @Get('/userView')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_groupAfterSale })
  async userListView(@Query() filter: QueryDTO_groupAfterSale, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const newData = [];
    for (const i of data) {
      const ad = await this.service.addInfo(i);
      const nd = new UserListView(ad);
      newData.push(nd);
    }
    const total = await this.service.count(filter);
    return { data: newData, total };
  }

  @Get('/canRefund/:id')
  async canRefund(@Param('id') id: string) {
    const order = await this.orderService.fetch(id);
    if (!order) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    const config = await this.configModel.findOne({}, { reward_day: 1 });
    const rd = _.get(config, 'reward_day', 14);
    const min = computedUtil.multiply(rd, computedUtil.multiply(24, 60));
    const buy_time = _.get(order, 'buy_time');
    const m = moment().diff(buy_time, 'm');
    const r = m < min;
    return r;
  }

  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
