import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { GroupOrderService } from '../service/groupOrder.service';
import {
  AdminListView,
  AdminView,
  CreateDTO_groupOrder,
  CreateVO_groupOrder,
  FetchVO_groupOrder,
  QueryDTO_groupOrder,
  QueryVO_groupOrder,
  toOrderPageDTO,
  UpdateDTO_groupOrder,
  UpdateVO_groupOrder,
  UserListView,
  UserView,
} from '../interface/groupOrder.interface';
import { ApiQuery, ApiResponse, ApiTags } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { RedisService } from '@midwayjs/redis';
import _ = require('lodash');
import { TransactionService } from '../util/transactions';
import { WxPayService } from '../util/wxpay';
import { GroupService } from '../service/group.service';
import { MqSender } from '../service/mq/mqSender.service';
import { ShopInBillService } from '../service/shopInBill.service';
@ApiTags(['团购订单'])
@Controller('/groupOrder')
export class GroupOrderController extends BaseController {
  @Inject()
  service: GroupOrderService;

  @Inject()
  redis: RedisService;

  @Inject()
  wxpayService: WxPayService;

  @Inject()
  groupService: GroupService;

  @Inject()
  mqSender: MqSender;

  @Inject()
  shopInBillService: ShopInBillService;

  @Post('/checkCanBuy')
  @Validate()
  async checkCanBuy(@Body() data: toOrderPageDTO, makeCache = true) {
    // 进入订单页前的检查
    // 1.检查店铺是否运行
    await this.service.checkCanBuy_shop(data);
    // 2.检查商品是否可以购买
    await this.service.checkCanBuy_goods(data);
    // 3.检查规格是否可以购买
    await this.service.checkCanBuy_goodsSpec(data);
    // 4.检查团是否满足可以够买
    const { result, msg } = await this.groupService.checkGroup(data.group);
    if (!result) throw new ServiceError(msg, FrameworkErrorEnum.SERVICE_FAULT);
    // 5.检查规格是否有团购设置
    await this.service.checkCanBuy_specInGroupConfig(data);
    // 检查完了,做缓存,将key返回回去
    if (makeCache) {
      const key = await this.service.makeCache(data);
      return key;
    }
    return 'ok';
  }

  @Post('/toOrderPage')
  async toOrderPage(@Body('key') key: string) {
    const checkData = await this.service.getCache(key);
    const { result, msg } = await this.groupService.checkGroup(checkData.group);
    if (!result) throw new ServiceError(msg, FrameworkErrorEnum.SERVICE_FAULT);
    const data = await this.service.getOrderPageData(checkData);
    return data;
  }

  @Post('/')
  // @Validate()
  @ApiResponse({ type: CreateVO_groupOrder })
  async create(@Body() data: CreateDTO_groupOrder) {
    // 需要重写,不是简单创建了
    const customer = _.get(this.ctx, 'user._id');
    if (!customer) throw new ServiceError('缺少用户信息', FrameworkErrorEnum.NEED_PARAMS);
    // 1.重走一遍商品检查
    const checkData: toOrderPageDTO = {
      shop: _.get(data, 'shop'),
      goods: _.get(data, 'goods'),
      goodsSpec: _.get(data, 'goodsSpec'),
      num: _.get(data, 'num'),
      group: _.get(data, 'group'),
    };
    await this.checkCanBuy(checkData, false);
    // 2.组织订单数据
    const createData = await this.service.toMakeOrderData(data, customer);
    // 3.事务保存
    const tran = new TransactionService();
    try {
      const id = tran.insert('GroupOrder', createData);
      // 4.减库存
      await this.service.dealGoods(createData, tran);
      await tran.run();
      // 5.创建死信
      await this.mqSender.orderMsg({ order_id: id });
      return id;
    } catch (error) {
      await tran.rollback();
    }
  }
  @Get('/')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_groupOrder })
  async query(@Query() filter: QueryDTO_groupOrder, @Query('skip') skip: number, @Query('limit') limit: number) {
    const data = await this.service.query(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const newData = [];
    for (const i of data) {
      const ni = await this.service.addInfo(i);
      const nd = new AdminListView(ni);
      newData.push(nd);
    }
    const total = await this.service.count(filter);
    return { data: newData, total };
  }

  @Get('/userView')
  @ApiQuery({
    name: 'query',
  })
  @ApiResponse({ type: QueryVO_groupOrder })
  async userView(@Query() filter: QueryDTO_groupOrder, @Query('skip') skip: number, @Query('limit') limit: number) {
    const { data, total } = await this.service.userListView(filter, { skip, limit, sort: { 'meta.createdAt': -1 } });
    const newData = [];
    for (const i of data) {
      const ni = await this.service.addInfo(i);
      const nd = new UserListView(ni);
      newData.push(nd);
    }
    return { data: newData, total };
  }

  @Get('/userView/:id')
  @ApiResponse({ type: FetchVO_groupOrder })
  async userFetch(@Param('id') id: string) {
    const data = await this.service.fetch(id, { populate: true });
    const result = new UserView(data);
    return result;
  }

  @Get('/:id')
  @ApiResponse({ type: AdminView })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id, { populate: true });
    const ni = await this.service.addInfo(data);
    const result = new AdminView(ni);
    return result;
  }

  @Post('/:id')
  @ApiResponse({ type: UpdateVO_groupOrder })
  async update(@Param('id') id: string, @Body() body: UpdateDTO_groupOrder) {
    const status = body.status;
    const groupSuccess = await this.service.checkGroupSuccess(id);
    if (!groupSuccess && parseInt(status) > 1) throw new ServiceError('该团未拼团成功,无法进行订单处理');
    const oldData = await this.service.fetch(id);
    let result;
    // 如果没有改状态,那就直接修改内容
    if (status === _.get(oldData, 'status')) result = await this.service.updateOne(id, body);
    else {
      // 如果改了状态,那就要看是哪种状态
      if (status === '3') {
        // 收货了,需要给团长记账了
        const tran = new TransactionService();
        try {
          tran.update('GroupOrder', { _id: id }, body);
          // 佣金金额; 店铺本单实际收入 =  订单实际支付价格*(1-抽成) - 佣金金额
          const commission = await this.service.leaderCommission(id, tran);
          await this.shopInBillService.createByOrder(id, commission, tran);
          await tran.run();
        } catch (error) {
          await tran.rollback();
          throw new ServiceError('收货发生错误', FrameworkErrorEnum.SERVICE_FAULT);
        }
      } else result = await this.service.updateOne(id, body);
    }
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }

  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
