import { Body, Controller, Inject, Post } from '@midwayjs/decorator';
import { ApiTags } from '@midwayjs/swagger';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { WxPayService } from '../util/wxpay';
import _ = require('lodash');
import { GroupOrderService } from '../service/groupOrder.service';
import * as computedUtil from '../util/computed';
import { PayData, WxRefundData } from '../interface/pay.interface';
import { User } from '../entity/base/user';
import { ReturnModelType } from '@typegoose/typegoose';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { TransactionService } from '../util/transactions';
import moment = require('moment');
import { GroupService } from '../service/group.service';
import { randomStr } from '../util/util';
@ApiTags(['订单支付相关'])
@Controller('/orderDeal')
export class PayController {
  @Inject()
  wxpayService: WxPayService;

  @Inject()
  orderService: GroupOrderService;
  @Inject()
  groupService: GroupService;
  @InjectEntityModel(User)
  userModel: ReturnModelType<typeof User>;

  @Post('/pay')
  /**订单支付 */
  async pay(@Body() body: object) {
    const order_id = _.get(body, 'order_id');
    if (!order_id) throw new ServiceError('缺少订单信息', FrameworkErrorEnum.NEED_BODY);
    const order = await this.orderService.fetch(order_id);
    if (!order) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 检查团是否可以继续购买
    const group = _.get(order, 'group._id');
    const { result, msg } = await this.groupService.checkGroup(group);
    if (!result) throw new ServiceError(msg, FrameworkErrorEnum.SERVICE_FAULT);
    if (_.get(order, 'pay.result')) throw new ServiceError('订单已支付', FrameworkErrorEnum.SERVICE_FAULT);
    let rePayTimes = 0;
    const pay = _.get(order, 'pay', {});
    let pay_no = _.get(pay, 'pay_no');
    if (pay_no) {
      const arr = pay_no.split('-');
      const last = _.last(arr);
      rePayTimes = computedUtil.plus(last, 1);
      // 关闭前一个订单
      await this.wxpayService.close(pay_no);
    }
    pay_no = `${_.get(order, 'no')}-${rePayTimes}`;
    pay.pay_no = pay_no;
    const customer = _.get(order, 'customer');
    if (!customer) throw new ServiceError('订单信息缺少购买人信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    const user = await this.userModel.findById(customer).lean();
    const payData: PayData = { money: _.get(order, 'pay.pay_money'), desc: '团购', order_no: pay_no, openid: _.get(user, 'openid') };
    await this.orderService.updateOne(_.get(order, '_id'), { pay });
    let data = await this.wxpayService.create(payData);
    data = this.wxpayService.preparToUniAppWxPay(data);
    return data;
  }

  /**订单取消 */
  @Post('/cancel')
  async cancel(@Body() body: object) {
    const order_id = _.get(body, 'order_id');
    if (!order_id) throw new ServiceError('缺少订单信息', FrameworkErrorEnum.NEED_BODY);
    const order = await this.orderService.fetch(order_id);
    if (!order) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    if (_.get(order, 'pay.result')) throw new ServiceError('该订单已支付完成', FrameworkErrorEnum.SERVICE_FAULT);
    if (_.get(order, 'status') !== '0') throw new ServiceError('该订单不处于可以退单的状态', FrameworkErrorEnum.SERVICE_FAULT);
    const tran = new TransactionService();
    try {
      // 加库存
      await this.orderService.returnGoods(order, tran);
      tran.update('GroupOrder', { _id: _.get(order, '_id') }, { status: '-1' });
      // 改状态
      await tran.run();
    } catch (error) {
      await tran.rollback();
      throw new ServiceError('订单取消失败', FrameworkErrorEnum.SERVICE_FAULT);
    }
  }

  /**支付回调 */
  @Post('/callback')
  async callback(@Body() body: object) {
    const result = _.get(body, 'result');
    if (!result) throw new ServiceError('缺少支付回调信息', FrameworkErrorEnum.NEED_BODY);
    const { out_trade_no, payer } = result;
    const openid = _.get(payer, 'openid');
    const query = { 'pay.pay_no': new RegExp(`${out_trade_no}`), 'pay.openid': openid };
    const order = await this.orderService.findOne(query);
    if (!order) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    const payData = _.get(order, 'pay', {});
    // 支付结果全都存起来
    payData.result = result;
    payData.pay_time = moment().format('YYYY-MM-DD HH:mm:ss');
    // 检查团状态
    const { result: gr, msg } = await this.groupService.checkGroup(order.group);
    if (!gr) {
      // 不满足团条件.退款
      const rd = new WxRefundData();
      rd.money = _.get(order, 'pay.pay_money');
      rd.order_no = _.get(order, 'pay.pay_no');
      rd.out_refund_no = `${_.get(order, 'no')}-r-${randomStr()}`;
      rd.reason = '团购未参与成功';
      await this.wxpayService.refund(rd);
      // 还原库存
      const tran = new TransactionService();
      this.orderService.returnGoods(order, tran);
      tran.update('GroupOrder', { _id: _.get(order, '_id') }, { status: '-1' });
      await tran.run();
      throw new ServiceError(msg, FrameworkErrorEnum.SERVICE_FAULT);
    }
    const tran = new TransactionService();
    try {
      // 1.修改状态及支付结果
      tran.update('GroupOrder', { _id: order._id }, { pay: payData, status: '1' });
      // 2.加销量
      await this.orderService.addSell(_.get(order, 'goods'), _.get(order, 'num'), tran);
      await tran.run();
    } catch (error) {
      await tran.rollback();
      throw new ServiceError('支付回调失败', FrameworkErrorEnum.SERVICE_FAULT);
    }
  }
}
