import { Inject, Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { Group } from '../entity/group/group';
import moment = require('moment');
import { GroupOrder } from '../entity/group/groupOrder';
import _ = require('lodash');
import { lookUp } from '../util/pipeline.util';
import { MqSender } from './mq/mqSender.service';
import { Admin } from '../entity/base/admin';
import { HttpServiceFactory } from '@midwayjs/axios';
import { PopulateOptions } from 'mongoose';
import * as computedUtil from '../util/computed';

type modelType = ReturnModelType<typeof Group>;
@Provide()
export class GroupService extends BaseService<modelType> {
  @InjectEntityModel(Group)
  model: modelType;
  @InjectEntityModel(GroupOrder)
  orderModel: ReturnModelType<typeof GroupOrder>;
  @InjectEntityModel(Admin)
  adminModel: ReturnModelType<typeof Admin>;
  @Inject()
  httpServiceFactory: HttpServiceFactory;
  @Inject()
  mqSender: MqSender;

  async toSendCustomerMsg(id) {
    const group = await this.model.findById(id);
    const os = await this.orderModel.find({ group: id, status: _.get(group, 'status', '1') }, { customer: 1, goods: 1 }).lean();
    const gn = _.get(_.head(os), 'goods.name');
    const customer = os.map(i => i.customer);
    const content = `您参加的团购,商品为: ${gn} 团已拼团${_.get(group, 'status') === '1' ? '成功' : '失败'}`;
    const obj = { customer, source: '2', source_id: id, time: moment().format('YYYY-MM-DD HH:mm:ss'), content };
    const axios = this.httpServiceFactory.get('base');
    await axios.post('/notice', obj);
  }

  /**发送系统消息 */
  async sendSystemMsg(id) {
    const refs = this.getRefs();
    const group = await this.model
      .findById(id, { leader: 1, goods: 1 })
      .populate(refs as PopulateOptions[])
      .lean();
    if (!group) return;
    const leader = _.get(group, 'leader');
    const customer = [];
    if (leader) customer.push(leader);
    else {
      // 没有团长,那就是管理员的,咋发还没想好,也先直接把管理员查出来塞进去吧
      const admin = await this.adminModel.findOne({ account: 'sadmin' });
      if (admin) customer.push(_.get(admin, '_id'));
    }
    if (customer.length <= 0) return;
    const obj = { customer, source: '2', source_id: id, time: moment().format('YYYY-MM-DD HH:mm:ss'), content: `由您主导的团购,商品: ${_.get(group, 'goods.name')} 已临近截止时间,请及时进行操作` };
    const axios = this.httpServiceFactory.get('base');
    await axios.post('/notice', obj);
  }

  /** mq检查是否关团 */
  async autoCheck(params) {
    const id = _.get(params, 'id');
    const start_time = _.get(params, 'start_time');
    const end_time = _.get(params, 'end_time');
    if (!id) throw new ServiceError('缺少团信息', FrameworkErrorEnum.NEED_PARAMS);
    const group = await this.model.findById(id).lean();
    if (!group) throw new ServiceError('未找到团信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    const ost = _.get(group, 'start_time');
    const oet = _.get(group, 'end_time');
    if (ost !== start_time || oet !== end_time) throw new ServiceError('时间已修改,此判断无效', FrameworkErrorEnum.SERVICE_FAULT);
    const status = _.get(group, 'status');
    // 团已结束,不需要处理
    if (status === '1') return;
    // 团已经散了,不需要处理
    else if (status === '-1') return;
    // 准备中,不需要处理
    else if (status === '2') return;
    return true;
  }
  /**判断是否需要mq信息 */
  async needMqMsg(data) {
    const id = _.get(data, 'id');
    const start_time = _.get(data, 'start_time');
    const end_time = _.get(data, 'end_time');
    const status = _.get(data, 'status');
    if (!(start_time && end_time)) return;
    if (!id) return;
    const group = await this.model.findById(id);
    if (!group) throw new ServiceError('未找到团信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 没有团信息,那就看原数据是什么
    if (!status) {
      const sg = _.get(group, 'status');
      // 如果原数据也不是开团中,那就不管了
      if (sg !== '0') return;
    }
    // 状态不是开团中,不需要管
    else if (status !== '0') return;
    const ost = _.get(group, 'start_time');
    const oet = _.get(group, 'end_time');
    if (ost !== start_time || oet !== end_time) {
      // 需要重新发送mq消息,之前的消息会被判断失败
      const msg = { id, start_time, end_time };
      return msg;
    }
  }
  /**发送mq消息 */
  async toSendMsg(data) {
    const start_time = _.get(data, 'start_time');
    const end_time = _.get(data, 'end_time');
    if (!(start_time && end_time)) return;
    // mq作用改变,从原来关闭团,变为提示,所以需要提前一些时间,暂定提前3分钟
    const ms = moment(end_time).subtract(3, 'm').diff(moment());
    const obj = _.pick(data, ['id', 'start_time', 'end_time']);
    if (!obj.id) obj.id = _.get(data, '_id');
    await this.mqSender.groupMsg(obj, ms);
  }

  /**
   * 检查当前团是否满足可以参团
   * @param id 团id
   */
  async checkGroup(id) {
    const g = await this.model.findById(id);
    const result = { result: true, msg: '' };
    // 找这个团
    if (!g) {
      result.result = false;
      result.msg = '未找到指定团';
      return result;
    }
    const { status, start_time, end_time, person_limit } = g;
    // 看这个团的状态
    if (status !== '0') {
      result.result = false;
      if (status === '2') result.msg = '团正在准备中';
      else if (status === '-1') result.msg = '已散团';
      else if (status === '1') result.msg = '团购结束';
      return result;
    }
    const r = moment().isBetween(start_time, end_time, null, '[]');
    // 再看当前时间
    if (!r) {
      result.result = false;
      result.msg = '当前时间不在该团开团时间内';
      return result;
    }
    // 再看已经有几单了
    const num = await this.orderModel.count({
      group: id,
      status: '1',
    });
    if (num >= person_limit) {
      result.result = false;
      result.msg = '团已满人';
      return result;
    }
    return result;
  }

  async userListView(filter, pageOptions = {}) {
    const dup = _.cloneDeep(filter.getFilter());
    const refs = this.getRefs();
    const pipeline = [];
    for (const ref of refs) {
      const path = _.get(ref, 'path');
      const modelName = _.lowerFirst(_.get(ref, 'model.modelName'));
      if (!path) continue;
      const arr = lookUp(path, modelName);
      if (path === 'leader') {
        arr.pop();
      }
      pipeline.push(...arr);
    }
    pipeline.push({ $match: dup });
    if (_.get(pageOptions, 'sort')) pipeline.push({ $sort: _.get(pageOptions, 'sort') });
    const qp = _.cloneDeep(pipeline);
    qp.push({ $addFields: { group: { $toString: '$_id' } } });
    qp.push({
      $lookup: {
        from: 'groupOrder',
        localField: 'group',
        foreignField: 'group',
        pipeline: [{ $match: { status: '1' } }, { $project: { _id: 1 } }],
        as: 'orders',
      },
    });
    qp.push({
      $addFields: {
        can_group: {
          $cond: {
            if: { $eq: ['$person_limit', { $size: '$orders' }] },
            then: true,
            else: false,
          },
        },
      },
    });
    if (_.get(pageOptions, 'skip')) qp.push({ $skip: _.get(pageOptions, 'skip') });
    if (_.get(pageOptions, 'limit')) qp.push({ $limit: _.get(pageOptions, 'limit') });
    const data = await this.model.aggregate(qp);
    const totalResult = await this.model.aggregate([...pipeline, { $count: 'total' }]);
    const total = _.get(_.head(totalResult), 'total', 0);
    return { data, total };
  }

  async aggQuery(filter, { skip, limit }) {
    const { shop, shop_name, goods, goods_name, leader, leader_name, status, start_time, end_time } = filter;
    const pipeline = [];
    const step1: any = {};
    if (shop) step1.shop = shop;
    if (goods) step1.goods = goods;
    if (leader) step1.leader = leader;
    if (status) step1.status = status;
    if (start_time) step1.start_time = { $gte: start_time };
    if (end_time) step1.end_time = { $lte: end_time };
    if (Object.keys(step1).length > 0) pipeline.push({ $match: step1 });
    const step2: any = [];
    const refs = this.getRefs();
    for (const ref of refs) {
      const path = _.get(ref, 'path');
      const modelName = _.lowerFirst(_.get(ref, 'model.modelName'));
      if (!path) continue;
      const arr = lookUp(path, modelName);
      step2.push(...arr);
    }
    pipeline.push(...step2);
    const step3: any = {};
    if (shop_name) step3['shop.name'] = new RegExp(shop_name);
    if (goods_name) step3['goods.name'] = new RegExp(goods_name);
    if (leader_name) step3['leader.name'] = new RegExp(leader_name);
    if (Object.keys(step3).length > 0) pipeline.push({ $match: step3 });
    const qp = _.cloneDeep(pipeline);
    if (limit) {
      qp.push({ $skip: parseInt(skip) });
      qp.push({ $limit: parseInt(limit) });
    }
    const data = await this.model.aggregate(qp);
    for (const i of data) {
      const gc = _.minBy(i.group_config, 'price');
      const group_price = _.get(gc, 'price', 0);
      i.group_price = computedUtil.toNumber(group_price);
      const sell_price = _.get(gc, 'spec.sell_money', 0);
      i.sell_price = computedUtil.toNumber(sell_price);
    }
    const tp = _.cloneDeep(pipeline);
    tp.push({ $count: 'total' });
    const tr = await this.model.aggregate(tp);
    const total = _.get(_.head(tr), 'total', 0);
    return { data, total };
  }
}
