import { Inject, Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { GroupAfterSale } from '../entity/group/groupAfterSale';
import { GroupOrder } from '../entity/group/groupOrder';
import { GroupOrderService } from './groupOrder.service';
import _ = require('lodash');
import * as computedUtil from '../util/computed';
import moment = require('moment');
import { CashBack } from '../entity/base/cashBack';
import { randomStr } from '../util/util';
import { Shop } from '../entity/base/shop';
import { ShopInBillService } from './shopInBill.service';
type modelType = ReturnModelType<typeof GroupAfterSale>;
@Provide()
export class GroupAfterSaleService extends BaseService<modelType> {
  @InjectEntityModel(GroupAfterSale)
  model: modelType;

  @InjectEntityModel(GroupOrder)
  orderModel: ReturnModelType<typeof GroupOrder>;

  @InjectEntityModel(Shop)
  shopModel: ReturnModelType<typeof Shop>;

  @Inject()
  groupOrderService: GroupOrderService;
  @InjectEntityModel(CashBack)
  cashBackModel: ReturnModelType<typeof CashBack>;

  @Inject()
  shopInBillService: ShopInBillService;

  async create(body: object): Promise<object> {
    const order = _.get(body, 'order');
    const type = _.get(body, 'type');
    // 1.校验数据是否存在
    const o = await this.groupOrderService.fetch(order);
    if (!o) throw new ServiceError('未找到订单信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 2.再校验订单是否在处理售后
    const hasData = await this.model.count({ order, status: { $nin: ['!1', '!2', '!3', '!4', '!5'] } });
    if (hasData > 0) throw new ServiceError('该商品已有正在处理中的售后申请.请勿重复申请', FrameworkErrorEnum.SERVICE_FAULT);
    if (type === '4' || type === '5') {
      const moneyDetail = await this.groupOrderService.moneyDetail(o);
      const realPay = _.get(moneyDetail, 'total');
      const obj = { money: realPay, desc: type === '4' ? '取消订单' : '拒收商品' };
      Object.assign(body, obj);
    }
    if (!_.get(body, 'shop')) Object.assign(body, { shop: _.get(o, 'shop._id', _.get(o, 'shop')) });
    const data = await this.model.create(body);
    return data;
  }

  refundOrder(id, tran) {
    tran.update('GroupOrder', { _id: id }, { status: '-4' });
  }
  /**
   * 整单退
   * @param data 售后数据
   * @param tran 事务实例
   */
  async returnOrder(data, tran) {
    const order_id = _.get(data, 'order');
    const order = await this.orderModel.findById(order_id);
    if (!order) throw new ServiceError('未找到订单数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    let returnMoney = _.get(data, 'money');
    if (!returnMoney || computedUtil.minus(_.get(data, 'money'), _.get(data, 'pay.pay_money')) < 0) returnMoney = _.get(data, 'pay.pay_money');
    const order_no = _.get(order, 'pay.pay_no');
    const str = randomStr();
    const out_refund_no = `${order_no}-r-${str}`;
    const refundInfo = { reason: _.get(data, 'desc', '购物退款'), money: returnMoney, order_no, out_refund_no };
    return refundInfo;
  }

  /**
   * 退钱
   * @param data 售后数据
   * @param body 要修改成的数据
   * @param tran 事务实例
   */
  async toReturnMoney(data, body, tran) {
    const order_id = _.get(data, 'order._id');
    const order = await this.orderModel.findById(order_id);
    if (!order) throw new ServiceError('未找到订单数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 组织退款信息
    let returnMoney = _.get(body, 'money');
    // 如果在要修改的数据中没有金额,那就直接去找原售后数据中的退款金额
    if (!returnMoney) returnMoney = _.get(data, 'money');
    // 再判断,退款金额是否超过支付金额
    const real_pay = _.get(order, 'pay.pay_money');
    // 退款金额超过实际支付金额,那就使用支付金额作为退款金额
    if (computedUtil.minus(returnMoney, real_pay) > 0) returnMoney = real_pay;
    const order_no = _.get(order, 'pay.pay_no');
    const str = randomStr();
    const out_refund_no = `${order_no}-r-${str}`;
    const refundInfo = { reason: _.get(data, 'desc', '购物退款'), money: returnMoney, order_no, out_refund_no };
    // 组织出账信息
    // 先找入账记录,有入账,再出账,否则干扣了
    // 记录退款金额
    let refundMoney = _.cloneDeep(returnMoney);
    const inBill = await this.cashBackModel.findOne({ source: '2', source_id: order._id });
    if (inBill) {
      const num = await this.cashBackModel.count({ source: '-2', source_id: data._id });
      if (num <= 0) {
        // 有入账信息,看入账信息中的佣金是否能抵消退款金额
        const get = _.get(inBill, 'money');
        // 团长需要出账的金额
        let needRefundForBill = 0;
        // 如果退款金额 比 团长入账的还多, 那就只能退团长入账的金额
        if (computedUtil.minus(refundMoney, get) > 0) needRefundForBill = get;
        // 反之,直接退
        else needRefundForBill = refundMoney;
        refundMoney = computedUtil.minus(refundMoney, get);
        const cashBackData = {
          inviter: _.get(inBill, 'inviter'),
          money: needRefundForBill,
          time: moment().format('YYYY-MM-DD HH:mm:ss'),
          status: '-1',
          source: '-2',
          source_id: data._id,
        };
        tran.insert('CashBack', cashBackData);
      }
    }
    if (refundMoney > 0) {
      // 团长的佣金退完了,但是退款的钱仍没有退完,需要从店铺中再退
      await this.shopInBillService.createByAfterSale(data, refundMoney, tran);
    }
    //
    return refundInfo;
  }

  async addInfo(data) {
    const shop = _.get(data, 'order.shop');
    if (!shop) return data;
    const sd = await this.shopModel.findById(shop).lean();
    if (sd) data.order.shop = sd;
    return data;
  }
}
