import { Provide } from '@midwayjs/core';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { ShopInBill } from '../entity/base/shopInBill';
import _ = require('lodash');
import { Shop } from '../entity/base/shop';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import moment = require('moment');
import * as computedUtil from '../util/computed';
import { GroupOrder } from '../entity/group/groupOrder';
@Provide()
export class ShopInBillService {
  @InjectEntityModel(ShopInBill)
  model: ReturnModelType<typeof ShopInBill>;
  @InjectEntityModel(Shop)
  shopModel: ReturnModelType<typeof Shop>;
  @InjectEntityModel(GroupOrder)
  orderModel: ReturnModelType<typeof GroupOrder>;

  /**
   * 收货创建店铺流水
   * 店铺本单实际收入 =  订单实际支付价格*(1-抽成) - 佣金金额
   * @param id 订单id
   * @param commission 佣金金额
   * @param tran 数据库事务
   */
  async createByOrder(id, commission, tran) {
    const order = await this.orderModel.findById(id).lean();
    if (!order) throw new ServiceError('未找到订单数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    const obj: any = { shop: _.get(order, 'shop') };
    const shopData = await this.shopModel.findById(_.get(order, 'shop'), { cut: 1 });
    if (!shopData) throw new ServiceError('未找到店铺数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    obj.cut = _.get(shopData, 'cut', 0);
    const real_pay = _.get(order, 'pay.pay_money');
    obj.total = real_pay;
    let receipts = computedUtil.multiply(computedUtil.minus(1, computedUtil.divide(obj.cut, 100)), real_pay);
    // 减佣金
    receipts = computedUtil.minus(receipts, commission);
    obj.receipts = receipts;
    obj.time = moment().format('YYYY-MM-DD HH:mm:ss');
    obj.source = '2';
    obj.source_id = _.get(order, '_id');
    const num = await this.model.count({ source: obj.source, source_id: obj.source_id });
    if (num <= 0) tran.insert('ShopInBill', obj);
  }

  /**
   * 退货创建店铺流水
   * @param afterSale 售后数据
   * @param returnMoney 还需退款金额
   * @param tran 数据库事务
   */
  async createByAfterSale(afterSale, returnMoney, tran) {
    if (returnMoney <= 0) return;
    const query: any = { source: '2' };
    if (_.isObject(_.get(afterSale, 'order'))) query.source_id = _.get(afterSale, 'order._id');
    else if (_.isString(_.get(afterSale, 'order'))) query.source_id = _.get(afterSale, 'order');
    const inBill = await this.model.findOne(query);
    if (!inBill) return;
    const q2 = { source: '-2', source_id: _.get(afterSale, '_id') };
    const n2 = await this.model.count(q2);
    if (n2 > 0) return;
    const obj: any = {};
    if (_.isObject(_.get(afterSale, 'shop'))) obj.shop = _.get(afterSale, 'shop._id');
    else if (_.isString(_.get(afterSale, 'shop'))) obj.shop = _.get(afterSale, 'shop');
    obj.time = moment().format('YYYY-MM-DD HH:mm:ss');
    obj.source = '-2';
    obj.source_id = _.get(afterSale, 'order');
    const limit = _.get(inBill, 'receipts');
    if (computedUtil.minus(limit, returnMoney) >= 0) obj.receipts = returnMoney;
    else obj.receipts = limit;
    tran.insert('ShopInBill', obj);
  }
}
