import { Provide, Inject, App } from '@midwayjs/decorator';
import { HttpServiceFactory } from '@midwayjs/axios';
import { Application } from '@midwayjs/koa';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import _ = require('lodash');
import { WxRefundData } from '../interface/pay.interface';
@Provide()
export class WxPayService {
  key = 'wechat';
  @App()
  app: Application;
  @Inject()
  httpServiceFactory: HttpServiceFactory;
  /**
   * 支付下单
   */
  async create(data) {
    const { money, openid, order_no, desc } = data;
    if (!money) throw new ServiceError('微信支付请求:缺少金额', FrameworkErrorEnum.NEED_BODY);
    if (!openid) throw new ServiceError('微信支付请求:缺少支付用户信息', FrameworkErrorEnum.NEED_BODY);
    if (!order_no) throw new ServiceError('微信支付请求:缺少订单号', FrameworkErrorEnum.NEED_BODY);
    const config = this.app.getConfig('wxPayConfig');
    const notice_url = this.app.getConfig('wxPayCallBack');
    const wxOrderData = { config, money, openid, order_no, desc, notice_url };
    const axios = await this.httpServiceFactory.get(this.key);
    const result = await axios.post('/pay/payOrder', wxOrderData);
    return _.get(result, 'data');
    // if (result.status === 200) {
    //   return _.get(result, 'data.data');
    // } else throw new ServiceError('微信支付请求失败', FrameworkErrorEnum.SERVICE_FAULT);
  }

  /**
   * 关闭支付订单
   */
  async close(order_no) {
    if (!order_no) throw new ServiceError('微信关闭订单请求:缺少订单号', FrameworkErrorEnum.NEED_BODY);
    const config = this.app.getConfig('wxPayConfig');
    const params = { config, order_no };
    const axios = await this.httpServiceFactory.get(this.key);
    const result = await axios.post('/pay/closeOrder', params);
    return _.get(result, 'data');
    // if (result.status === 200) {
    //   return _.get(result, 'data.data');
    // } else throw new ServiceError('微信关闭订单请求失败', FrameworkErrorEnum.SERVICE_FAULT);
  }

  /**
   * 查询支付单
   */
  async search(order_no) {
    if (!order_no) throw new ServiceError('微信关闭订单请求:缺少订单号', FrameworkErrorEnum.NEED_BODY);
    const config = this.app.getConfig('wxPayConfig');
    const params = { config, order_no };
    const axios = await this.httpServiceFactory.get(this.key);
    const result = await axios.post('/pay/searchOrderByOrderNo', params);
    return _.get(result, 'data');
    // if (result.status === 200) {
    //   return _.get(result, 'data.data');
    // } else throw new ServiceError('微信关闭订单请求失败', FrameworkErrorEnum.SERVICE_FAULT);
  }

  /**
   * 支付单退款
   */
  async refund(data: WxRefundData) {
    const { order_no, out_refund_no, money, reason } = data;
    if (!order_no) throw new ServiceError('微信退款请求:缺少订单号', FrameworkErrorEnum.NEED_BODY);
    if (!out_refund_no) throw new ServiceError('微信退款请求:缺少退款单号', FrameworkErrorEnum.NEED_BODY);
    const config = this.app.getConfig('wxPayConfig');
    const params = { config, order_no, out_refund_no, money, reason };
    const axios = await this.httpServiceFactory.get(this.key);
    const result = await axios.post('/pay/refundOrder', params);
    return _.get(result, 'data');
    // if (result.status === 200) {
    //   return _.get(result, 'data.data');
    // } else throw new ServiceError('微信退款请求失败', FrameworkErrorEnum.SERVICE_FAULT);
  }
  /**
   * 组织uniapp的微信支付数据
   * @param data 微信请求支付返回值
   */
  preparToUniAppWxPay(data: object): object {
    const obj = {
      nonceStr: _.get(data, 'nonceStr'),
      package: `prepay_id=${_.get(data, 'prepay_id')}`,
      signType: _.get(data, 'signType'),
      timeStamp: _.get(data, 'timestamp'),
      paySign: _.get(data, 'paySign'),
    };
    return obj;
  }
}
